<?php

if ( ! defined( 'ABSPATH' ) ) {
        exit; // Exit if accessed directly.
}

// An example for an extra stripe payment gateway
// In this example the internal name is stripe2, many of the functions below have this in their name
// The filters eme_payment_gateways, eme_configured_payment_gateways are always needed
// The filter eme_payment_gateway_change_total is only needed for gateways that need cents or any other price conversion
// The filter eme_payment_gateway_extra_cost is optional if extra cost is needed, also change MY_STRIPE2_EXTRA_CHARGE then
// The action hook eme_options_postsave_action is needed for those gateways that need a webhook that can't be created dynamically
//   (currently only needed for stripe)
// The functions needed (the name of the function ends on the internal name for the payment gateway):
//    eme_payment_form_stripe2
//    eme_complete_transaction_stripe2
//    eme_notification_stripe2
//    eme_charge_stripe2
//    eme_stripe2_webhook (this is an internal function used by the action hook eme_options_postsave_action)
// Optional refund function
//    eme_refund_booking_stripe2

define ('MY_STRIPE2_PRIVATE_KEY','sk_test_D...'); // sk_...
define ('MY_STRIPE2_PUBLIC_KEY','pk_test_v...'); // pk_...
define ('MY_STRIPE2_PAYMENT_METHODS','card'); // comma-separated list of wanted methods (see dropdown of original stripe method for values)
define ('MY_STRIPE2_BUTTON_ABOVE_TEXT','');
define ('MY_STRIPE2_BUTTON_BELOW_TEXT','');
define ('MY_STRIPE2_BUTTON_LABEL','Stripe 2'); // the text on the payment button
define ('MY_STRIPE2_BUTTON_IMAGE_URL','');
define ('MY_STRIPE2_EXTRA_CHARGE','');
define ('MY_STRIPE2_EXTRA_CHARGE2','');
define ('MY_STRIPE2_PG_NAME','Stripe 2'); // the name of the payment gateway shown in admin overview/dropdown

add_filter('eme_payment_gateways',"my_extra_pgs");
function my_extra_pgs($pgs) {
	$pgs['stripe2'] = MY_STRIPE2_PG_NAME;
	return $pgs;
}

add_filter('eme_configured_payment_gateways',"my_extra_configured_pgs");
function my_extra_configured_pgs($pgs) {
	$pgs[] = 'stripe2';
	return $pgs;
}

add_filter('eme_payment_gateway_change_total',"my_extra_pgs_change_total", 10, 2);
function my_extra_pgs_change_total($price, $gateway ) {
	if ( $gateway == 'stripe2' ) {
		$price *= 100; //stripe actually wants cents
	}
	return $price;
}

add_filter('eme_payment_gateway_extra_cost',"my_extra_pgs_extra_cost", 10, 2);
function my_extra_pgs_extra_cost($price, $gateway ) {
	$result = 0;
	if ( $gateway == 'stripe2' ) {
                if (!empty(MY_STRIPE2_EXTRA_CHARGE)) {
                        $extra = MY_STRIPE2_EXTRA_CHARGE;
                        if ( strstr( $extra, '%' ) ) {
                                $extra   = str_replace( '%', '', $extra );
                                $result += sprintf( '%01.2f', $price * $extra / 100 );
                        } else {
                                $result += sprintf( '%01.2f', $extra );
                        }
                }
                if (!empty(MY_STRIPE2_EXTRA_CHARGE2)) {
                        $extra = MY_STRIPE2_EXTRA_CHARGE2;
                        if ( strstr( $extra, '%' ) ) {
                                $extra   = str_replace( '%', '', $extra );
                                $result += sprintf( '%01.2f', $price * $extra / 100 );
                        } else {
                                $result += sprintf( '%01.2f', $extra );
                        }
                }
	}
	return $result;
}

// stripe is about the only one that needs a hook created like this ...
// This is not needed for other payment gateways
add_action('eme_options_postsave_action',"my_options_postsave_action");
function my_options_postsave_action() {
        $tab = isset( $_GET['tab'] ) ? eme_sanitize_request( $_GET['tab'] ) : 'general';
        // if we saved settings on the payments tab, certain webhooks need to be created
        if ( $tab == 'payments' ) {
                eme_stripe2_webhook();
        }
}

// now the code for stripe2 itself, which is basically a copy of the stripe code with stripe=>stripe2
function eme_payment_form_stripe2( $item_name, $payment, $baseprice, $cur, $multi_booking = 0 ) {
        $gateway               = 'stripe2';
        $eme_stripe_public_key = MY_STRIPE2_PUBLIC_KEY;
        if ( ! $eme_stripe_public_key ) {
                return;
        }

        $description = eme_get_payment_desc( $item_name, $payment, $gateway, $multi_booking );

        // gateway doesn't like the single quotes 
        $description = str_replace( "'", '', $description );
        $description = eme_esc_html( $description );

        $price            = eme_payment_gateway_total( $baseprice, $cur, $gateway );
        $events_page_link = eme_get_events_page();
        $payment_id       = $payment['id'];

        $button_above = MY_STRIPE2_BUTTON_ABOVE_TEXT;
        $button_label = MY_STRIPE2_BUTTON_LABEL;
        if ( empty( $button_label ) ) {
                $button_label = $gateway;
        }
        $button_label   = htmlentities( $button_label );
        $button_below   = MY_STRIPE2_BUTTON_BELOW_TEXT;
        $button_img_url = MY_STRIPE2_BUTTON_IMAGE_URL;

        $form_html  = $button_above;
        $form_html .= "<form action='' method='post' name='eme_{$gateway}_form' id='eme_{$gateway}_form'>
   <input type='hidden' name='payment_id' value='$payment_id'>
   <input type='hidden' name='eme_eventAction' value='{$gateway}_charge'>
   <input type='hidden' name='description' value='$description'>
   <input type='hidden' name='price' value='$price'>
   <input type='hidden' name='cur' value='$cur'>
   ";
        if ( ! empty( $button_img_url ) ) {
                $form_html .= "<input type='image' src='$button_img_url' alt='$button_label' title='$button_label' class='button-primary eme_submit_button'><br>";
        } else {
                $form_html .= "<input type='submit' value='$button_label' class='button-primary eme_submit_button'><br>";
        }
        $form_html .= wp_nonce_field( "$price$cur", "eme_{$gateway}_nonce", false, false );
        $form_html .= '</form>';
        $form_html .= $button_below;
        return $form_html;
}

function eme_complete_transaction_stripe2( $payment ) {
	$gateway = 'stripe2';
	$eme_plugin_dir  = eme_plugin_dir();
        require_once $eme_plugin_dir.'/payment_gateways/stripe/init.php';
        $eme_stripe_private_key = MY_STRIPE2_PRIVATE_KEY;
        \Stripe\Stripe::setApiKey( "$eme_stripe_private_key" );

        $stripe_sessionid = $payment['pg_pid'];
        $stripe_session   = \Stripe\Checkout\Session::retrieve( $stripe_sessionid ); 
        // we don't trust the payment given, we use the one in the session from stripe
        $payment_id   = $stripe_session->client_reference_id;
        $stripe_pi_id = $stripe_session->payment_intent;
        if ( $payment_id == $payment['id'] ) {
                try {
                        $stripe_pi = \Stripe\PaymentIntent::retrieve( $stripe_pi_id );
                } catch ( Exception $e ) {
                        return;
                }
                if ( $stripe_pi->status == 'succeeded' ) {
                        // we store the payment intent id, so we can refund later
                        eme_mark_payment_paid( $payment_id, 1, $gateway, $stripe_pi_id );
                }
        }
}

function eme_notification_stripe2() {
	$gateway = 'stripe2';
        $eme_stripe_private_key = MY_STRIPE2_PRIVATE_KEY;
        $webhook_secret         = get_option( 'eme_stripe2_webhook_secret' );

	$eme_plugin_dir  = eme_plugin_dir();
        require_once $eme_plugin_dir.'/payment_gateways/stripe/init.php';
        \Stripe\Stripe::setApiKey( "$eme_stripe_private_key" );

        $payload    = @file_get_contents( 'php://input' );
        $sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
        $event      = null;

        // verify the signature
        try {
                $event = \Stripe\Webhook::constructEvent(
                    $payload,
                    $sig_header,
                    $webhook_secret
                );
        } catch ( \UnexpectedValueException $e ) {
                // Invalid payload
                http_response_code( 400 );
                return;
        } catch ( \Stripe\Exception\SignatureVerificationException $e ) {
                // Invalid signature
                http_response_code( 400 );
                return;
        }

        // Handle the checkout.session.completed event
        if ( $event->type == 'checkout.session.completed' ) {
                $stripe_session = $event->data->object;
                $payment_id     = $stripe_session->client_reference_id;
                $stripe_pi_id   = $stripe_session->payment_intent;
                eme_mark_payment_paid( $payment_id, 1, $gateway, $stripe_pi_id );
        }
        http_response_code( 200 );
}

// only for stripe 
function eme_stripe2_webhook() {
	$gateway = "stripe2";
        $eme_stripe_private_key = MY_STRIPE2_PRIVATE_KEY;
        if ( empty( $eme_stripe_private_key ) ) {
                return;
        }
        // do nothing if the events page is on localhost
        $events_page_link  = eme_get_events_page();
        $notification_link = add_query_arg( [ 'eme_eventAction' => "{$gateway}_notification" ], $events_page_link );
        if ( strstr( $events_page_link, 'localhost' ) ) {
                return;
        }

	$eme_plugin_dir  = eme_plugin_dir();
        require_once $eme_plugin_dir.'/payment_gateways/stripe/init.php';
        \Stripe\Stripe::setApiKey( "$eme_stripe_private_key" );

        // first check all webhooks, delete the one matching our url and recreate it, otherwise we can't get the secret
        $webhooks = null;
        try {
                $webhooks = \Stripe\WebhookEndpoint::all( [ 'limit' => 100 ] );
                if ( ! empty( $webhooks ) ) {
                        foreach ( $webhooks->data as $webhook ) {
                                $endpoint_id = $webhook->id;
                                $endpoint    = \Stripe\WebhookEndpoint::retrieve( "$endpoint_id" );
                                if ( $endpoint->url == $notification_link ) {
                                        $endpoint->delete();
                                }
                        }
                }
        } catch ( \Stripe\Exception\InvalidRequestException $e ) {
                return;
        }

        update_option( 'eme_stripe2_webhook_secret', '' );
        try {
                $endpoint = \Stripe\WebhookEndpoint::create(
                        [
                                'url'            => $notification_link,
                                'enabled_events' => [ 'checkout.session.completed' ],
                        ]
                );
        } catch ( \Stripe\Exception\InvalidRequestException $e ) {
                return;
        }
        update_option( 'eme_stripe2_webhook_secret', $endpoint->secret );
}

function eme_charge_stripe2() {
	$gateway = "stripe2";
        $events_page_link = eme_get_events_page();
        $payment_id       = intval( $_POST['payment_id'] );
        $price            = eme_sanitize_request( $_POST['price'] );
        $cur              = eme_sanitize_request( $_POST['cur'] );
        $description      = eme_sanitize_request( $_POST['description'] );
        $payment          = eme_get_payment( $payment_id );
        $success_link     = eme_payment_return_url( $payment, $gateway );
        $fail_link        = eme_payment_return_url( $payment, 1 );

        // no cheating
        if ( ! wp_verify_nonce( eme_sanitize_request($_POST["eme_{$gateway}_nonce"]), "$price$cur" ) ) {
		return eme_js_redirect($fail_link);
        }

        // avoid that people pay again after pressing "back" and arriving on the payment form again
        $check_allowed_to_pay = eme_payment_allowed_to_pay( $payment_id );
        if ( ! empty( $check_allowed_to_pay )) {
                // not allowed: return the reason and stop
                return $check_allowed_to_pay;
        }

        $eme_stripe_private_key = MY_STRIPE2_PRIVATE_KEY;
        $eme_stripe_public_key = MY_STRIPE2_PUBLIC_KEY;
        $item_name              = esc_attr( get_option( 'blog_name' ) );
        if ( empty( $item_name ) ) {
                $item_name = $description;
        }

	$eme_plugin_dir  = eme_plugin_dir();
        require_once $eme_plugin_dir.'/payment_gateways/stripe/init.php';
        \Stripe\Stripe::setApiKey( "$eme_stripe_private_key" );
        \Stripe\Stripe::setAppInfo( 'WordPress Events Made Easy Stripe plugin' );

        $payment_methods = MY_STRIPE2_PAYMENT_METHODS;
        if ( empty( $payment_methods ) ) {
                $payment_methods = 'card';
        }
        // make sure it is an array with numeric keys
        if ( ! is_array( $payment_methods ) ) {
                $payment_methods_arr = explode( ',', $payment_methods );
        } else {
                $payment_methods_arr = $payment_methods;
        }
        $stripe_session_params = [
                'payment_method_types' => $payment_methods_arr,
                'payment_intent_data'  => [ 'description' => $description ],
                'line_items'           => [
                        [
                                'price_data' => [
                                        'currency'     => strtolower( $cur ),
                                        'unit_amount'  => $price,
                                        'product_data' => [
                                                'name'        => $item_name,
                                                'description' => $description,
                                        ],
                                ],
                                'quantity'   => 1,
                        ],
                ],
                'mode'                 => 'payment',
                'client_reference_id'  => $payment_id,
                'success_url'          => $success_link,
                'cancel_url'           => $fail_link,
        ];

        // prefill e-mail in Stripe form if possible
        $booking_ids = eme_get_payment_booking_ids( $payment_id );
        if ( $booking_ids ) {
                $booking = eme_get_booking( $booking_ids[0] );
                $person  = eme_get_person( $booking['person_id'] );
                if (!empty($person)) {
                        $stripe_session_params['customer_email'] = $person['email'];
                }
        }

        $stripe_session = \Stripe\Checkout\Session::create($stripe_session_params);

        $stripe_session_id = $stripe_session->id;
        eme_update_payment_pg_pid( $payment_id, $stripe_session_id );

        print "<html><body>
<script src='https://js.stripe.com/v3/'></script>
<script>
      var stripe = Stripe('$eme_stripe_public_key');
      stripe.redirectToCheckout({
         sessionId: '$stripe_session_id'
      });
</script>
</body></html>
   ";
}

// optional refund function
// if not present, no refund will happen of course
function eme_refund_booking_stripe2( $booking ) {
        $gateway = 'stripe2';

        $eme_stripe_private_key = MY_STRIPE2_PRIVATE_KEY;
        if ( ! $eme_stripe_private_key ) {
                return;
        }

	$eme_plugin_dir  = eme_plugin_dir();
        require_once $eme_plugin_dir.'/payment_gateways/stripe/init.php';
        \Stripe\Stripe::setApiKey( "$eme_stripe_private_key" );

        $stripe_pi_id = $booking['pg_pid'];
        try {
                $stripe_pi = \Stripe\PaymentIntent::retrieve( $stripe_pi_id );
        } catch ( Exception $e ) {
                return;
        }
        $re = \Stripe\Refund::create(
            [
                        'payment_intent' => $stripe_pi_id,
                ]
        );
        return true;
}

